------------------------------------------------------------------------------
--  This file is a part of the LPP VHDL IP LIBRARY
--  Copyright (C) 2009 - 2010, Laboratory of Plasmas Physic - CNRS
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more Cdetails.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
-------------------------------------------------------------------------------
--                    Author : Paul LEROY
--                    Mail   : paul.leroy@lpp.polytechnique.fr
-------------------------------------------------------------------------------

LIBRARY IEEE;
USE IEEE.numeric_std.ALL;
USE IEEE.std_logic_1164.ALL;
LIBRARY staging_lpp;
USE staging_lpp.PLE_general_purpose.ALL;

ENTITY BUTTERFLY_DATAFLOW IS
  GENERIC (
    Sample_SZ    : INTEGER := 16
    );
  PORT (
    rstn : IN STD_LOGIC;
    clk  : IN STD_LOGIC;

    Are            : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    Aim            : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    Bre            : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    Bim            : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    c_in           : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    cps_in         : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    cms_in         : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);

    op1            : OUT STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    op2            : OUT STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);

    out_alu        : OUT STD_LOGIC_VECTOR ( 2*Sample_SZ-1 DOWNTO 0);

    sel_op1        : IN STD_LOGIC_VECTOR( 4 DOWNTO 0 ); -- Are Aim X Y Z
    sel_op2        : IN STD_LOGIC_VECTOR( 4 DOWNTO 0 ); -- Bre Bim C_in cps_in cms_in
    sel_xyz        : IN STD_LOGIC_VECTOR( 2 DOWNTO 0 ); -- X Y Z
    alu_ctrl       : IN STD_LOGIC_VECTOR( 2 DOWNTO 0 );
    alu_comp       : IN STD_LOGIC_VECTOR( 1 DOWNTO 0 )
    );
END BUTTERFLY_DATAFLOW;

ARCHITECTURE ar_BUTTERFLY_DATAFLOW OF BUTTERFLY_DATAFLOW IS

    SIGNAL X : STD_LOGIC_VECTOR( Sample_SZ-1 DOWNTO 0 );
    SIGNAL Y : STD_LOGIC_VECTOR( Sample_SZ-1 DOWNTO 0 );
    SIGNAL Z : STD_LOGIC_VECTOR( Sample_SZ-1 DOWNTO 0 );

    SIGNAL ALU_OP1 : STD_LOGIC_VECTOR( Sample_SZ-1 DOWNTO 0 );
    SIGNAL ALU_OP2 : STD_LOGIC_VECTOR( Sample_SZ-1 DOWNTO 0 );

    SIGNAL OUT_ALU_SIG  : STD_LOGIC_VECTOR ( 2*Sample_SZ-1 DOWNTO 0);

BEGIN

out_alu <= OUT_ALU_SIG;

PROCESS (clk, rstn)

    BEGIN  -- PROCESS
        IF rstn = '0' THEN                  -- asynchronous reset (active low)
            X                  <= (OTHERS => '0');
            Y                  <= (OTHERS => '0');
            Z                  <= (OTHERS => '0');
      
        ELSIF clk'EVENT AND clk = '1' THEN  -- rising clock edge  

            if sel_xyz = "100" THEN
                X <= STD_LOGIC_VECTOR(resize(SIGNED(OUT_ALU_SIG), Sample_SZ));
            elsif sel_xyz = "010" THEN
                Y <= STD_LOGIC_VECTOR(resize(SIGNED(OUT_ALU_SIG), Sample_SZ));
            elsif sel_xyz = "001" THEN
                Z <= STD_LOGIC_VECTOR(resize(SIGNED(OUT_ALU_SIG), Sample_SZ));
            end if;

        END IF;

    END PROCESS;

    op1 <= ALU_OP1;
    op2 <= ALU_OP2;

ALU_OP1 <=  Are  WHEN sel_op1 = "10000" ELSE
            Aim  WHEN sel_op1 = "01000" ELSE
            X    WHEN sel_op1 = "00100" ELSE
            Y    WHEN sel_op1 = "00010" ELSE
            Z    WHEN sel_op1 = "00001" ELSE
            STD_LOGIC_VECTOR(resize(SIGNED(OUT_ALU_SIG), Sample_SZ)) WHEN sel_op1 = "00000" ELSE
            (OTHERS => '0');

ALU_OP2 <=  Bre     WHEN sel_op2 = "10000" ELSE
            Bim     WHEN sel_op2 = "01000" ELSE
            c_in    WHEN sel_op2 = "00100" ELSE
            cps_in  WHEN sel_op2 = "00010" ELSE
            cms_in  WHEN sel_op2 = "00001" ELSE
            std_logic_vector(TO_SIGNED(1,Sample_SZ)) WHEN sel_op2 = "00000" ELSE
            (OTHERS => '0');

ALU_1: ALU
    GENERIC MAP (
        Arith_en   => 1,
        Input_SZ_1 => Sample_SZ,
        Input_SZ_2 => Sample_SZ,
        COMP_EN    => 0) -- comp is enable when COMP_EN is 0
    PORT MAP (
        clk   => clk,
        reset => rstn,
        ctrl  => alu_ctrl,
        comp  => alu_comp,
        OP1   => ALU_OP1,
        OP2   => ALU_OP2,
        RES   => OUT_ALU_SIG);

END ar_BUTTERFLY_DATAFLOW;